# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/appointment_template.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2
from omni.pro.protos.v1.rules import delivery_method_pb2 as v1_dot_rules_dot_delivery__method__pb2
from omni.pro.protos.v1.rules import holidays_pb2 as v1_dot_rules_dot_holidays__pb2
from omni.pro.protos.v1.rules import warehouse_pb2 as v1_dot_rules_dot_warehouse__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n#v1/rules/appointment_template.proto\x12.pro.omni.oms.api.v1.rules.appointment_template\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1ev1/rules/delivery_method.proto\x1a\x18v1/rules/warehouse.proto\x1a\x17v1/rules/holidays.proto"d\n\x18\x41ppointmentTemplateLines\x12\x0b\n\x03\x64\x61y\x18\x01 \x01(\t\x12\x12\n\nhour_start\x18\x02 \x01(\t\x12\x10\n\x08hour_end\x18\x03 \x01(\t\x12\x15\n\rorder_numbers\x18\x04 \x01(\x05"\xab\x04\n\x13\x41ppointmentTemplate\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x42\n\nwarehouses\x18\x03 \x03(\x0b\x32..pro.omni.oms.api.v1.rules.warehouse.Warehouse\x12J\n\x07methods\x18\x04 \x03(\x0b\x32\x39.pro.omni.oms.api.v1.rules.delivery_method.DeliveryMethod\x12\x15\n\rorder_numbers\x18\x05 \x01(\x05\x12\x1b\n\x13hour_limit_same_day\x18\x06 \x01(\t\x12\x1b\n\x13number_days_to_show\x18\x07 \x01(\x05\x12>\n\x08holidays\x18\x08 \x01(\x0b\x32,.pro.omni.oms.api.v1.rules.holidays.Holidays\x12W\n\x05lines\x18\t \x03(\x0b\x32H.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateLines\x12*\n\x06\x61\x63tive\x18\n \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x0b \x01(\t\x12?\n\x0cobject_audit\x18\x0c \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xe6\x02\n AppointmentTemplateCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rwarehouse_ids\x18\x02 \x03(\x05\x12\x12\n\nmethod_ids\x18\x03 \x03(\t\x12\x15\n\rorder_numbers\x18\x04 \x01(\x05\x12\x1b\n\x13hour_limit_same_day\x18\x05 \x01(\t\x12\x1b\n\x13number_days_to_show\x18\x06 \x01(\x05\x12\x12\n\nholiday_id\x18\x07 \x01(\t\x12W\n\x05lines\x18\x08 \x03(\x0b\x32H.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateLines\x12\x13\n\x0b\x65xternal_id\x18\t \x01(\t\x12\x36\n\x07\x63ontext\x18\n \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd1\x01\n!AppointmentTemplateCreateResponse\x12\x61\n\x14\x61ppointment_template\x18\x01 \x01(\x0b\x32\x43.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplate\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xfc\x02\n\x1e\x41ppointmentTemplateReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x8b\x02\n\x1f\x41ppointmentTemplateReadResponse\x12\x62\n\x15\x61ppointment_templates\x18\x01 \x03(\x0b\x32\x43.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplate\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xbd\x01\n AppointmentTemplateUpdateRequest\x12\x61\n\x14\x61ppointment_template\x18\x01 \x01(\x0b\x32\x43.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplate\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd1\x01\n!AppointmentTemplateUpdateResponse\x12\x61\n\x14\x61ppointment_template\x18\x01 \x01(\x0b\x32\x43.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplate\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"f\n AppointmentTemplateDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"n\n!AppointmentTemplateDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xaa\x06\n\x1a\x41ppointmentTemplateService\x12\xc2\x01\n\x19\x41ppointmentTemplateCreate\x12P.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateCreateRequest\x1aQ.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateCreateResponse"\x00\x12\xbc\x01\n\x17\x41ppointmentTemplateRead\x12N.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateReadRequest\x1aO.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateReadResponse"\x00\x12\xc2\x01\n\x19\x41ppointmentTemplateUpdate\x12P.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateUpdateRequest\x1aQ.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateUpdateResponse"\x00\x12\xc2\x01\n\x19\x41ppointmentTemplateDelete\x12P.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateDeleteRequest\x1aQ.pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.appointment_template_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_APPOINTMENTTEMPLATELINES"]._serialized_start = 221
    _globals["_APPOINTMENTTEMPLATELINES"]._serialized_end = 321
    _globals["_APPOINTMENTTEMPLATE"]._serialized_start = 324
    _globals["_APPOINTMENTTEMPLATE"]._serialized_end = 879
    _globals["_APPOINTMENTTEMPLATECREATEREQUEST"]._serialized_start = 882
    _globals["_APPOINTMENTTEMPLATECREATEREQUEST"]._serialized_end = 1240
    _globals["_APPOINTMENTTEMPLATECREATERESPONSE"]._serialized_start = 1243
    _globals["_APPOINTMENTTEMPLATECREATERESPONSE"]._serialized_end = 1452
    _globals["_APPOINTMENTTEMPLATEREADREQUEST"]._serialized_start = 1455
    _globals["_APPOINTMENTTEMPLATEREADREQUEST"]._serialized_end = 1835
    _globals["_APPOINTMENTTEMPLATEREADRESPONSE"]._serialized_start = 1838
    _globals["_APPOINTMENTTEMPLATEREADRESPONSE"]._serialized_end = 2105
    _globals["_APPOINTMENTTEMPLATEUPDATEREQUEST"]._serialized_start = 2108
    _globals["_APPOINTMENTTEMPLATEUPDATEREQUEST"]._serialized_end = 2297
    _globals["_APPOINTMENTTEMPLATEUPDATERESPONSE"]._serialized_start = 2300
    _globals["_APPOINTMENTTEMPLATEUPDATERESPONSE"]._serialized_end = 2509
    _globals["_APPOINTMENTTEMPLATEDELETEREQUEST"]._serialized_start = 2511
    _globals["_APPOINTMENTTEMPLATEDELETEREQUEST"]._serialized_end = 2613
    _globals["_APPOINTMENTTEMPLATEDELETERESPONSE"]._serialized_start = 2615
    _globals["_APPOINTMENTTEMPLATEDELETERESPONSE"]._serialized_end = 2725
    _globals["_APPOINTMENTTEMPLATESERVICE"]._serialized_start = 2728
    _globals["_APPOINTMENTTEMPLATESERVICE"]._serialized_end = 3538
# @@protoc_insertion_point(module_scope)



import json

import fakeredis
import redis
from omni.pro.config import Config
from omni.pro.util import nested


class RedisConnection:
    def __init__(self, host: str, port: int, db: int) -> None:
        self.host = host
        self.port = int(port)
        self.db = db

    def __enter__(self) -> redis.StrictRedis:
        self.redis_client = redis.StrictRedis(
            host=self.host, port=self.port, db=self.db, decode_responses=True, ssl=Config.REDIS_SSL
        )
        return self.redis_client

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.redis_client.close()


class RedisManager(object):
    def __init__(self, host: str, port: int, db: int) -> None:
        self.host = host
        self.port = int(port)
        self.db = db
        self._connection = RedisConnection(host=self.host, port=self.port, db=self.db)

    def get_connection(self) -> RedisConnection:
        if Config.TESTING:
            return fakeredis.FakeStrictRedis(
                server=FakeRedisServer.get_instance(),
                charset="utf-8",
                decode_responses=True,
            )
        return self._connection

    def set_connection(self, connection: RedisConnection) -> None:
        self._connection = connection

    def set_json(self, key, json_obj):
        with self.get_connection() as rc:
            if isinstance(json_obj, str):
                json_obj = json.loads(json_obj)
            return rc.json().set(key, "$", json_obj)

    def get_json(self, key, *args, no_escape=False):
        with self.get_connection() as rc:
            return rc.json().get(key, *args, no_escape=no_escape)

    def get_resource_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_json(tenant_code)
        # logger.info(f"Redis config", extra={"deb_config": config})
        return {
            **nested(config, f"resources.{service_id}", {}),
            **nested(config, "aws", {}),
        }

    def get_aws_cognito_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "region_name": nested(config, "aws.cognito.region"),
            "aws_access_key_id": config.get("aws_access_key_id"),
            "aws_secret_access_key": config.get("aws_secret_access_key"),
            "user_pool_id": nested(config, "aws.cognito.user_pool_id"),
            "client_id": nested(config, "aws.cognito.client_id"),
        }

    def get_aws_s3_config(self, service_id: str, tenant_code: str) -> dict:
        """
        Retrieves the configuration settings for an AWS S3 service based on a given service ID and tenant code.

        :param service_id: str
        The unique identifier for the service.

        :param tenant_code: str
        The code representing the tenant for which the configuration is required.

        :return: dict
        Returns a dictionary containing the S3 configuration, including region name, access key ID, secret access key, and bucket name.

        The method retrieves the configuration using `get_resource_config` and extracts S3-specific settings such as the region, access keys, and bucket name.
        """
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "region_name": nested(config, "aws.s3.region"),
            "aws_access_key_id": config.get("aws_access_key_id"),
            "aws_secret_access_key": config.get("aws_secret_access_key"),
            "bucket_name": nested(config, "aws.s3.bucket_name"),
            "allowed_files": nested(config, "aws.s3.allowed_files") or [],
        }

    def get_mongodb_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "host": nested(config, "dbs.mongodb.host"),
            "port": nested(config, "dbs.mongodb.port"),
            "user": nested(config, "dbs.mongodb.user"),
            "password": nested(config, "dbs.mongodb.pass"),
            "name": nested(config, "dbs.mongodb.name"),
            "complement": nested(config, "dbs.mongodb.complement"),
        }

    def get_postgres_config(self, service_id: str, tenant_code: str) -> dict:
        config = self.get_resource_config(service_id, tenant_code)
        return {
            "host": nested(config, "dbs.postgres.host"),
            "port": nested(config, "dbs.postgres.port"),
            "user": nested(config, "dbs.postgres.user"),
            "password": nested(config, "dbs.postgres.pass"),
            "name": nested(config, "dbs.postgres.name"),
        }

    def get_tenant_codes(self, pattern="*", exlcudes_keys=["SETTINGS"]) -> list:
        with self.get_connection() as rc:
            if Config.REDIS_SSL is False:
                return [key for key in rc.keys(pattern=pattern) if key not in exlcudes_keys]

            cursor = "0"
            keys = []
            while cursor != 0:
                cursor, next_keys = rc.scan(cursor=cursor, match=pattern)
                keys.extend(next_keys)
            return [key for key in keys if key not in exlcudes_keys]

    def get_user_admin(self, tenant):
        tenant_obj = self.get_json(tenant)
        return tenant_obj.get("user_admin") or {}

    def get_load_balancer_config(self, service_id, tennat):
        config = self.get_resource_config(service_id, tennat)
        return {
            "host": nested(config, "load_balancer"),
            "port": nested(config, "port"),
        }

    def get_load_balancer_name(self, service_id, tennat):
        config = self.get_load_balancer_config(service_id, tennat)
        return f"{config.get('host')}:{config.get('port')}"


class FakeRedisServer:
    _instance = None

    @classmethod
    def get_instance(cls) -> fakeredis.FakeServer:
        if not cls._instance:
            cls._instance = cls._create_instance()
        return cls._instance

    @classmethod
    def _create_instance(cls) -> fakeredis.FakeServer:
        server = fakeredis.FakeServer()
        return server



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import appointment_template_pb2 as v1_dot_rules_dot_appointment__template__pb2


class AppointmentTemplateServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AppointmentTemplateCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateCreate",
            request_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateCreateResponse.FromString,
        )
        self.AppointmentTemplateRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateRead",
            request_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateReadResponse.FromString,
        )
        self.AppointmentTemplateUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateUpdate",
            request_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateUpdateResponse.FromString,
        )
        self.AppointmentTemplateDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateDelete",
            request_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateDeleteResponse.FromString,
        )


class AppointmentTemplateServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AppointmentTemplateCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentTemplateRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentTemplateUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def AppointmentTemplateDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_AppointmentTemplateServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "AppointmentTemplateCreate": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateCreate,
            request_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateCreateResponse.SerializeToString,
        ),
        "AppointmentTemplateRead": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateRead,
            request_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateReadResponse.SerializeToString,
        ),
        "AppointmentTemplateUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateUpdate,
            request_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateUpdateResponse.SerializeToString,
        ),
        "AppointmentTemplateDelete": grpc.unary_unary_rpc_method_handler(
            servicer.AppointmentTemplateDelete,
            request_deserializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class AppointmentTemplateService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AppointmentTemplateCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateCreate",
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateCreateRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentTemplateRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateRead",
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateReadRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentTemplateUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateUpdate",
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateUpdateRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def AppointmentTemplateDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.appointment_template.AppointmentTemplateService/AppointmentTemplateDelete",
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateDeleteRequest.SerializeToString,
            v1_dot_rules_dot_appointment__template__pb2.AppointmentTemplateDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from omni.pro.protos.common import base_pb2 as _base_pb2
from omni.pro.protos.v1.rules import delivery_method_pb2 as _delivery_method_pb2
from omni.pro.protos.v1.rules import warehouse_pb2 as _warehouse_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class AppointmentLines(_message.Message):
    __slots__ = ["hour_start", "hour_end", "order_numbers", "appointment_available", "cids", "id"]
    HOUR_START_FIELD_NUMBER: _ClassVar[int]
    HOUR_END_FIELD_NUMBER: _ClassVar[int]
    ORDER_NUMBERS_FIELD_NUMBER: _ClassVar[int]
    APPOINTMENT_AVAILABLE_FIELD_NUMBER: _ClassVar[int]
    CIDS_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    hour_start: str
    hour_end: str
    order_numbers: int
    appointment_available: int
    cids: _containers.RepeatedScalarFieldContainer[str]
    id: str
    def __init__(
        self,
        hour_start: _Optional[str] = ...,
        hour_end: _Optional[str] = ...,
        order_numbers: _Optional[int] = ...,
        appointment_available: _Optional[int] = ...,
        cids: _Optional[_Iterable[str]] = ...,
        id: _Optional[str] = ...,
    ) -> None: ...

class Appointment(_message.Message):
    __slots__ = ["id", "warehouse", "method", "date", "lines", "active", "external_id", "object_audit"]
    ID_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    METHOD_FIELD_NUMBER: _ClassVar[int]
    DATE_FIELD_NUMBER: _ClassVar[int]
    LINES_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    OBJECT_AUDIT_FIELD_NUMBER: _ClassVar[int]
    id: str
    warehouse: _warehouse_pb2.Warehouse
    method: _delivery_method_pb2.DeliveryMethod
    date: str
    lines: _containers.RepeatedCompositeFieldContainer[AppointmentLines]
    active: _wrappers_pb2.BoolValue
    external_id: str
    object_audit: _base_pb2.ObjectAudit
    def __init__(
        self,
        id: _Optional[str] = ...,
        warehouse: _Optional[_Union[_warehouse_pb2.Warehouse, _Mapping]] = ...,
        method: _Optional[_Union[_delivery_method_pb2.DeliveryMethod, _Mapping]] = ...,
        date: _Optional[str] = ...,
        lines: _Optional[_Iterable[_Union[AppointmentLines, _Mapping]]] = ...,
        active: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        external_id: _Optional[str] = ...,
        object_audit: _Optional[_Union[_base_pb2.ObjectAudit, _Mapping]] = ...,
    ) -> None: ...

class AppointmentCreateRequest(_message.Message):
    __slots__ = ["warehouse_id", "method_id", "date", "lines", "external_id", "context"]
    WAREHOUSE_ID_FIELD_NUMBER: _ClassVar[int]
    METHOD_ID_FIELD_NUMBER: _ClassVar[int]
    DATE_FIELD_NUMBER: _ClassVar[int]
    LINES_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    warehouse_id: int
    method_id: str
    date: str
    lines: _containers.RepeatedCompositeFieldContainer[AppointmentLines]
    external_id: str
    context: _base_pb2.Context
    def __init__(
        self,
        warehouse_id: _Optional[int] = ...,
        method_id: _Optional[str] = ...,
        date: _Optional[str] = ...,
        lines: _Optional[_Iterable[_Union[AppointmentLines, _Mapping]]] = ...,
        external_id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AppointmentCreateResponse(_message.Message):
    __slots__ = ["appointment", "response_standard"]
    APPOINTMENT_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    appointment: Appointment
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        appointment: _Optional[_Union[Appointment, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AppointmentReadRequest(_message.Message):
    __slots__ = ["group_by", "sort_by", "fields", "filter", "paginated", "id", "context"]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    SORT_BY_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    PAGINATED_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    group_by: _containers.RepeatedCompositeFieldContainer[_base_pb2.GroupBy]
    sort_by: _base_pb2.SortBy
    fields: _base_pb2.Fields
    filter: _base_pb2.Filter
    paginated: _base_pb2.Paginated
    id: str
    context: _base_pb2.Context
    def __init__(
        self,
        group_by: _Optional[_Iterable[_Union[_base_pb2.GroupBy, _Mapping]]] = ...,
        sort_by: _Optional[_Union[_base_pb2.SortBy, _Mapping]] = ...,
        fields: _Optional[_Union[_base_pb2.Fields, _Mapping]] = ...,
        filter: _Optional[_Union[_base_pb2.Filter, _Mapping]] = ...,
        paginated: _Optional[_Union[_base_pb2.Paginated, _Mapping]] = ...,
        id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AppointmentReadResponse(_message.Message):
    __slots__ = ["appointments", "meta_data", "response_standard"]
    APPOINTMENTS_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    appointments: _containers.RepeatedCompositeFieldContainer[Appointment]
    meta_data: _base_pb2.MetaData
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        appointments: _Optional[_Iterable[_Union[Appointment, _Mapping]]] = ...,
        meta_data: _Optional[_Union[_base_pb2.MetaData, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AppointmentUpdateRequest(_message.Message):
    __slots__ = ["appointment", "context"]
    APPOINTMENT_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    appointment: Appointment
    context: _base_pb2.Context
    def __init__(
        self,
        appointment: _Optional[_Union[Appointment, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AppointmentUpdateResponse(_message.Message):
    __slots__ = ["appointment", "response_standard"]
    APPOINTMENT_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    appointment: Appointment
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        appointment: _Optional[_Union[Appointment, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AppointmentDeleteRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: str
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[str] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class AppointmentDeleteResponse(_message.Message):
    __slots__ = ["response_standard"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    def __init__(self, response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...) -> None: ...

class SearchAppointmentRequest(_message.Message):
    __slots__ = ["cids", "context"]
    CIDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    cids: _containers.RepeatedScalarFieldContainer[str]
    context: _base_pb2.Context
    def __init__(
        self, cids: _Optional[_Iterable[str]] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class SearchAppointmentResponse(_message.Message):
    __slots__ = ["data", "response_standard"]
    DATA_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    data: _struct_pb2.Struct
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        data: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class ConfirmAppointmentRequest(_message.Message):
    __slots__ = ["aid", "cid", "context"]
    AID_FIELD_NUMBER: _ClassVar[int]
    CID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    aid: str
    cid: str
    context: _base_pb2.Context
    def __init__(
        self,
        aid: _Optional[str] = ...,
        cid: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class ConfirmAppointmentResponse(_message.Message):
    __slots__ = ["success", "message", "response_standard"]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    success: bool
    message: str
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        success: bool = ...,
        message: _Optional[str] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...



import gettext


def set_language(language_code="es", localedir: str = None, domain="messages"):
    translation = gettext.translation(domain, localedir, languages=[language_code])
    translation.install()
    global _
    _ = translation.gettext
    return _



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/users/log_session.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2
from omni.pro.protos.v1.users import user_pb2 as v1_dot_users_dot_user__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1av1/users/log_session.proto\x12%pro.omni.oms.api.v1.users.log_session\x1a\x11\x63ommon/base.proto\x1a\x13v1/users/user.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xfe\x01\n\nLogSession\x12\n\n\x02id\x18\x01 \x01(\t\x12\x32\n\x04user\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.v1.users.user.User\x12\x11\n\toperation\x18\x03 \x01(\t\x12.\n\ndate_start\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x64\x61te_end\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12?\n\x0cobject_audit\x18\x06 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xf3\x02\n\x15LogSessionReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe7\x01\n\x16LogSessionReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12G\n\x0clog_sessions\x18\x02 \x03(\x0b\x32\x31.pro.omni.oms.api.v1.users.log_session.LogSession\x12\x39\n\tmeta_data\x18\x03 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData"g\n\x13PausedMethodRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe4\x01\n\x14PausedMethodResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x46\n\x0blog_session\x18\x02 \x01(\x0b\x32\x31.pro.omni.oms.api.v1.users.log_session.LogSession\x12\x39\n\tmeta_data\x18\x03 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData2\xb1\x02\n\x11LogSessionService\x12\x8f\x01\n\x0eLogSessionRead\x12<.pro.omni.oms.api.v1.users.log_session.LogSessionReadRequest\x1a=.pro.omni.oms.api.v1.users.log_session.LogSessionReadResponse"\x00\x12\x89\x01\n\x0cPausedMethod\x12:.pro.omni.oms.api.v1.users.log_session.PausedMethodRequest\x1a;.pro.omni.oms.api.v1.users.log_session.PausedMethodResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.users.log_session_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_LOGSESSION"]._serialized_start = 143
    _globals["_LOGSESSION"]._serialized_end = 397
    _globals["_LOGSESSIONREADREQUEST"]._serialized_start = 400
    _globals["_LOGSESSIONREADREQUEST"]._serialized_end = 771
    _globals["_LOGSESSIONREADRESPONSE"]._serialized_start = 774
    _globals["_LOGSESSIONREADRESPONSE"]._serialized_end = 1005
    _globals["_PAUSEDMETHODREQUEST"]._serialized_start = 1007
    _globals["_PAUSEDMETHODREQUEST"]._serialized_end = 1110
    _globals["_PAUSEDMETHODRESPONSE"]._serialized_start = 1113
    _globals["_PAUSEDMETHODRESPONSE"]._serialized_end = 1341
    _globals["_LOGSESSIONSERVICE"]._serialized_start = 1344
    _globals["_LOGSESSIONSERVICE"]._serialized_end = 1649
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/clients/client.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2
from omni.pro.protos.v1.clients import address_pb2 as v1_dot_clients_dot_address__pb2
from omni.pro.protos.v1.clients import type_document_pb2 as v1_dot_clients_dot_type__document__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x17v1/clients/client.proto\x12"pro.omni.oms.api.v1.clients.client\x1a\x11\x63ommon/base.proto\x1a\x18v1/clients/address.proto\x1a\x1ev1/clients/type_document.proto\x1a\x1cgoogle/protobuf/struct.proto"\x90\x03\n\x06\x43lient\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12N\n\rtype_document\x18\x03 \x01(\x0b\x32\x37.pro.omni.oms.api.v1.clients.type_document.TypeDocument\x12\x10\n\x08\x64ocument\x18\x04 \x01(\t\x12\x0e\n\x06mobile\x18\x05 \x01(\t\x12\r\n\x05phone\x18\x06 \x01(\t\x12\r\n\x05\x65mail\x18\x07 \x01(\t\x12\x35\n\x07\x63ountry\x18\x08 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Object\x12?\n\taddresses\x18\t \x03(\x0b\x32,.pro.omni.oms.api.v1.clients.address.Address\x12\x0e\n\x06\x61\x63tive\x18\n \x01(\x08\x12\x13\n\x0b\x65xternal_id\x18\x0b \x01(\t\x12?\n\x0cobject_audit\x18\x0c \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\x81\x02\n\x13\x43lientCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x18\n\x10type_document_id\x18\x02 \x01(\t\x12\x10\n\x08\x64ocument\x18\x03 \x01(\t\x12\x0e\n\x06mobile\x18\x04 \x01(\t\x12\r\n\x05phone\x18\x05 \x01(\t\x12\r\n\x05\x65mail\x18\x06 \x01(\t\x12\x35\n\x07\x63ountry\x18\x07 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Object\x12\x13\n\x0b\x65xternal_id\x18\x08 \x01(\t\x12\x36\n\x07\x63ontext\x18\t \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9d\x01\n\x14\x43lientCreateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12:\n\x06\x63lient\x18\x02 \x01(\x0b\x32*.pro.omni.oms.api.v1.clients.client.Client"\xef\x02\n\x11\x43lientReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd7\x01\n\x12\x43lientReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12;\n\x07\x63lients\x18\x03 \x03(\x0b\x32*.pro.omni.oms.api.v1.clients.client.Client"\x89\x01\n\x13\x43lientUpdateRequest\x12:\n\x06\x63lient\x18\x01 \x01(\x0b\x32*.pro.omni.oms.api.v1.clients.client.Client\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9d\x01\n\x14\x43lientUpdateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12:\n\x06\x63lient\x18\x02 \x01(\x0b\x32*.pro.omni.oms.api.v1.clients.client.Client"Y\n\x13\x43lientDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"a\n\x14\x43lientDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xdd\x01\n\x17\x43lientSyncByHashRequest\x12%\n\x04info\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x31\n\x10shipping_address\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x30\n\x0f\x62illing_address\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x93\x01\n\x18\x43lientSyncByHashResponse\x12,\n\x0b\x64\x61ta_client\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xb3\x05\n\x0e\x43lientsService\x12\x83\x01\n\x0c\x43lientCreate\x12\x37.pro.omni.oms.api.v1.clients.client.ClientCreateRequest\x1a\x38.pro.omni.oms.api.v1.clients.client.ClientCreateResponse"\x00\x12}\n\nClientRead\x12\x35.pro.omni.oms.api.v1.clients.client.ClientReadRequest\x1a\x36.pro.omni.oms.api.v1.clients.client.ClientReadResponse"\x00\x12\x83\x01\n\x0c\x43lientUpdate\x12\x37.pro.omni.oms.api.v1.clients.client.ClientUpdateRequest\x1a\x38.pro.omni.oms.api.v1.clients.client.ClientUpdateResponse"\x00\x12\x83\x01\n\x0c\x43lientDelete\x12\x37.pro.omni.oms.api.v1.clients.client.ClientDeleteRequest\x1a\x38.pro.omni.oms.api.v1.clients.client.ClientDeleteResponse"\x00\x12\x8f\x01\n\x10\x43lientSyncByHash\x12;.pro.omni.oms.api.v1.clients.client.ClientSyncByHashRequest\x1a<.pro.omni.oms.api.v1.clients.client.ClientSyncByHashResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.clients.client_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_CLIENT"]._serialized_start = 171
    _globals["_CLIENT"]._serialized_end = 571
    _globals["_CLIENTCREATEREQUEST"]._serialized_start = 574
    _globals["_CLIENTCREATEREQUEST"]._serialized_end = 831
    _globals["_CLIENTCREATERESPONSE"]._serialized_start = 834
    _globals["_CLIENTCREATERESPONSE"]._serialized_end = 991
    _globals["_CLIENTREADREQUEST"]._serialized_start = 994
    _globals["_CLIENTREADREQUEST"]._serialized_end = 1361
    _globals["_CLIENTREADRESPONSE"]._serialized_start = 1364
    _globals["_CLIENTREADRESPONSE"]._serialized_end = 1579
    _globals["_CLIENTUPDATEREQUEST"]._serialized_start = 1582
    _globals["_CLIENTUPDATEREQUEST"]._serialized_end = 1719
    _globals["_CLIENTUPDATERESPONSE"]._serialized_start = 1722
    _globals["_CLIENTUPDATERESPONSE"]._serialized_end = 1879
    _globals["_CLIENTDELETEREQUEST"]._serialized_start = 1881
    _globals["_CLIENTDELETEREQUEST"]._serialized_end = 1970
    _globals["_CLIENTDELETERESPONSE"]._serialized_start = 1972
    _globals["_CLIENTDELETERESPONSE"]._serialized_end = 2069
    _globals["_CLIENTSYNCBYHASHREQUEST"]._serialized_start = 2072
    _globals["_CLIENTSYNCBYHASHREQUEST"]._serialized_end = 2293
    _globals["_CLIENTSYNCBYHASHRESPONSE"]._serialized_start = 2296
    _globals["_CLIENTSYNCBYHASHRESPONSE"]._serialized_end = 2443
    _globals["_CLIENTSSERVICE"]._serialized_start = 2446
    _globals["_CLIENTSSERVICE"]._serialized_end = 3137
# @@protoc_insertion_point(module_scope)



from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from omni.pro.protos.common import base_pb2 as _base_pb2
from omni.pro.protos.v1.rules import delivery_method_pb2 as _delivery_method_pb2
from omni.pro.protos.v1.rules import holidays_pb2 as _holidays_pb2
from omni.pro.protos.v1.rules import warehouse_pb2 as _warehouse_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class AppointmentTemplateLines(_message.Message):
    __slots__ = ["day", "hour_start", "hour_end", "order_numbers"]
    DAY_FIELD_NUMBER: _ClassVar[int]
    HOUR_START_FIELD_NUMBER: _ClassVar[int]
    HOUR_END_FIELD_NUMBER: _ClassVar[int]
    ORDER_NUMBERS_FIELD_NUMBER: _ClassVar[int]
    day: str
    hour_start: str
    hour_end: str
    order_numbers: int
    def __init__(
        self,
        day: _Optional[str] = ...,
        hour_start: _Optional[str] = ...,
        hour_end: _Optional[str] = ...,
        order_numbers: _Optional[int] = ...,
    ) -> None: ...

class AppointmentTemplate(_message.Message):
    __slots__ = [
        "id",
        "name",
        "warehouses",
        "methods",
        "order_numbers",
        "hour_limit_same_day",
        "number_days_to_show",
        "holidays",
        "lines",
        "active",
        "external_id",
        "object_audit",
    ]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSES_FIELD_NUMBER: _ClassVar[int]
    METHODS_FIELD_NUMBER: _ClassVar[int]
    ORDER_NUMBERS_FIELD_NUMBER: _ClassVar[int]
    HOUR_LIMIT_SAME_DAY_FIELD_NUMBER: _ClassVar[int]
    NUMBER_DAYS_TO_SHOW_FIELD_NUMBER: _ClassVar[int]
    HOLIDAYS_FIELD_NUMBER: _ClassVar[int]
    LINES_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    OBJECT_AUDIT_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    warehouses: _containers.RepeatedCompositeFieldContainer[_warehouse_pb2.Warehouse]
    methods: _containers.RepeatedCompositeFieldContainer[_delivery_method_pb2.DeliveryMethod]
    order_numbers: int
    hour_limit_same_day: str
    number_days_to_show: int
    holidays: _holidays_pb2.Holidays
    lines: _containers.RepeatedCompositeFieldContainer[AppointmentTemplateLines]
    active: _wrappers_pb2.BoolValue
    external_id: str
    object_audit: _base_pb2.ObjectAudit
    def __init__(
        self,
        id: _Optional[str] = ...,
        name: _Optional[str] = ...,
        warehouses: _Optional[_Iterable[_Union[_warehouse_pb2.Warehouse, _Mapping]]] = ...,
        methods: _Optional[_Iterable[_Union[_delivery_method_pb2.DeliveryMethod, _Mapping]]] = ...,
        order_numbers: _Optional[int] = ...,
        hour_limit_same_day: _Optional[str] = ...,
        number_days_to_show: _Optional[int] = ...,
        holidays: _Optional[_Union[_holidays_pb2.Holidays, _Mapping]] = ...,
        lines: _Optional[_Iterable[_Union[AppointmentTemplateLines, _Mapping]]] = ...,
        active: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        external_id: _Optional[str] = ...,
        object_audit: _Optional[_Union[_base_pb2.ObjectAudit, _Mapping]] = ...,
    ) -> None: ...

class AppointmentTemplateCreateRequest(_message.Message):
    __slots__ = [
        "name",
        "warehouse_ids",
        "method_ids",
        "order_numbers",
        "hour_limit_same_day",
        "number_days_to_show",
        "holiday_id",
        "lines",
        "external_id",
        "context",
    ]
    NAME_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_IDS_FIELD_NUMBER: _ClassVar[int]
    METHOD_IDS_FIELD_NUMBER: _ClassVar[int]
    ORDER_NUMBERS_FIELD_NUMBER: _ClassVar[int]
    HOUR_LIMIT_SAME_DAY_FIELD_NUMBER: _ClassVar[int]
    NUMBER_DAYS_TO_SHOW_FIELD_NUMBER: _ClassVar[int]
    HOLIDAY_ID_FIELD_NUMBER: _ClassVar[int]
    LINES_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    name: str
    warehouse_ids: _containers.RepeatedScalarFieldContainer[int]
    method_ids: _containers.RepeatedScalarFieldContainer[str]
    order_numbers: int
    hour_limit_same_day: str
    number_days_to_show: int
    holiday_id: str
    lines: _containers.RepeatedCompositeFieldContainer[AppointmentTemplateLines]
    external_id: str
    context: _base_pb2.Context
    def __init__(
        self,
        name: _Optional[str] = ...,
        warehouse_ids: _Optional[_Iterable[int]] = ...,
        method_ids: _Optional[_Iterable[str]] = ...,
        order_numbers: _Optional[int] = ...,
        hour_limit_same_day: _Optional[str] = ...,
        number_days_to_show: _Optional[int] = ...,
        holiday_id: _Optional[str] = ...,
        lines: _Optional[_Iterable[_Union[AppointmentTemplateLines, _Mapping]]] = ...,
        external_id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AppointmentTemplateCreateResponse(_message.Message):
    __slots__ = ["appointment_template", "response_standard"]
    APPOINTMENT_TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    appointment_template: AppointmentTemplate
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        appointment_template: _Optional[_Union[AppointmentTemplate, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AppointmentTemplateReadRequest(_message.Message):
    __slots__ = ["group_by", "sort_by", "fields", "filter", "paginated", "id", "context"]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    SORT_BY_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    PAGINATED_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    group_by: _containers.RepeatedCompositeFieldContainer[_base_pb2.GroupBy]
    sort_by: _base_pb2.SortBy
    fields: _base_pb2.Fields
    filter: _base_pb2.Filter
    paginated: _base_pb2.Paginated
    id: str
    context: _base_pb2.Context
    def __init__(
        self,
        group_by: _Optional[_Iterable[_Union[_base_pb2.GroupBy, _Mapping]]] = ...,
        sort_by: _Optional[_Union[_base_pb2.SortBy, _Mapping]] = ...,
        fields: _Optional[_Union[_base_pb2.Fields, _Mapping]] = ...,
        filter: _Optional[_Union[_base_pb2.Filter, _Mapping]] = ...,
        paginated: _Optional[_Union[_base_pb2.Paginated, _Mapping]] = ...,
        id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AppointmentTemplateReadResponse(_message.Message):
    __slots__ = ["appointment_templates", "meta_data", "response_standard"]
    APPOINTMENT_TEMPLATES_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    appointment_templates: _containers.RepeatedCompositeFieldContainer[AppointmentTemplate]
    meta_data: _base_pb2.MetaData
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        appointment_templates: _Optional[_Iterable[_Union[AppointmentTemplate, _Mapping]]] = ...,
        meta_data: _Optional[_Union[_base_pb2.MetaData, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AppointmentTemplateUpdateRequest(_message.Message):
    __slots__ = ["appointment_template", "context"]
    APPOINTMENT_TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    appointment_template: AppointmentTemplate
    context: _base_pb2.Context
    def __init__(
        self,
        appointment_template: _Optional[_Union[AppointmentTemplate, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AppointmentTemplateUpdateResponse(_message.Message):
    __slots__ = ["appointment_template", "response_standard"]
    APPOINTMENT_TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    appointment_template: AppointmentTemplate
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        appointment_template: _Optional[_Union[AppointmentTemplate, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class AppointmentTemplateDeleteRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: str
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[str] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class AppointmentTemplateDeleteResponse(_message.Message):
    __slots__ = ["response_standard"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    def __init__(self, response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...) -> None: ...



from enum import Enum

from .mongo import DatabaseManager, DBUtil, MongoConnection, PolishNotationToMongoDB
from .postgres import PolishNotationToSQLAlchemy, PostgresDatabaseManager, QueryBuilder, SessionManager


class PersistenceTypeEnum(Enum):
    SQL = "SQL"
    NO_SQL = "NO_SQL"
    REDIS = "REDIS"

    def __str__(self) -> str:
        return self.value



from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from omni.pro.protos.common import base_pb2 as _base_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class Country(_message.Message):
    __slots__ = [
        "id",
        "code",
        "name",
        "phone_number_size",
        "phone_prefix",
        "require_zipcode",
        "currencies",
        "document_types",
        "territory_matrixes",
        "meta_data",
        "timezones",
        "languages",
        "low_level",
        "icon",
        "active",
        "external_id",
        "object_audit",
    ]
    ID_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PHONE_NUMBER_SIZE_FIELD_NUMBER: _ClassVar[int]
    PHONE_PREFIX_FIELD_NUMBER: _ClassVar[int]
    REQUIRE_ZIPCODE_FIELD_NUMBER: _ClassVar[int]
    CURRENCIES_FIELD_NUMBER: _ClassVar[int]
    DOCUMENT_TYPES_FIELD_NUMBER: _ClassVar[int]
    TERRITORY_MATRIXES_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    TIMEZONES_FIELD_NUMBER: _ClassVar[int]
    LANGUAGES_FIELD_NUMBER: _ClassVar[int]
    LOW_LEVEL_FIELD_NUMBER: _ClassVar[int]
    ICON_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    OBJECT_AUDIT_FIELD_NUMBER: _ClassVar[int]
    id: str
    code: str
    name: str
    phone_number_size: int
    phone_prefix: str
    require_zipcode: _wrappers_pb2.BoolValue
    currencies: _struct_pb2.ListValue
    document_types: _struct_pb2.ListValue
    territory_matrixes: _struct_pb2.ListValue
    meta_data: _struct_pb2.Struct
    timezones: _struct_pb2.ListValue
    languages: _struct_pb2.ListValue
    low_level: str
    icon: bytes
    active: _wrappers_pb2.BoolValue
    external_id: str
    object_audit: _base_pb2.ObjectAudit
    def __init__(
        self,
        id: _Optional[str] = ...,
        code: _Optional[str] = ...,
        name: _Optional[str] = ...,
        phone_number_size: _Optional[int] = ...,
        phone_prefix: _Optional[str] = ...,
        require_zipcode: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        currencies: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        document_types: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        territory_matrixes: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        meta_data: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
        timezones: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        languages: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        low_level: _Optional[str] = ...,
        icon: _Optional[bytes] = ...,
        active: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        external_id: _Optional[str] = ...,
        object_audit: _Optional[_Union[_base_pb2.ObjectAudit, _Mapping]] = ...,
    ) -> None: ...

class CountryCreateRequest(_message.Message):
    __slots__ = [
        "code",
        "name",
        "phone_number_size",
        "phone_prefix",
        "require_zipcode",
        "currencies_ids",
        "document_types_ids",
        "territory_matrixes_ids",
        "meta_data",
        "timezones_ids",
        "languages_ids",
        "low_level",
        "icon",
        "external_id",
        "context",
    ]
    CODE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    PHONE_NUMBER_SIZE_FIELD_NUMBER: _ClassVar[int]
    PHONE_PREFIX_FIELD_NUMBER: _ClassVar[int]
    REQUIRE_ZIPCODE_FIELD_NUMBER: _ClassVar[int]
    CURRENCIES_IDS_FIELD_NUMBER: _ClassVar[int]
    DOCUMENT_TYPES_IDS_FIELD_NUMBER: _ClassVar[int]
    TERRITORY_MATRIXES_IDS_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    TIMEZONES_IDS_FIELD_NUMBER: _ClassVar[int]
    LANGUAGES_IDS_FIELD_NUMBER: _ClassVar[int]
    LOW_LEVEL_FIELD_NUMBER: _ClassVar[int]
    ICON_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    code: str
    name: str
    phone_number_size: int
    phone_prefix: str
    require_zipcode: _wrappers_pb2.BoolValue
    currencies_ids: _struct_pb2.ListValue
    document_types_ids: _struct_pb2.ListValue
    territory_matrixes_ids: _struct_pb2.ListValue
    meta_data: _struct_pb2.Struct
    timezones_ids: _struct_pb2.ListValue
    languages_ids: _struct_pb2.ListValue
    low_level: str
    icon: bytes
    external_id: str
    context: _base_pb2.Context
    def __init__(
        self,
        code: _Optional[str] = ...,
        name: _Optional[str] = ...,
        phone_number_size: _Optional[int] = ...,
        phone_prefix: _Optional[str] = ...,
        require_zipcode: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        currencies_ids: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        document_types_ids: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        territory_matrixes_ids: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        meta_data: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ...,
        timezones_ids: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        languages_ids: _Optional[_Union[_struct_pb2.ListValue, _Mapping]] = ...,
        low_level: _Optional[str] = ...,
        icon: _Optional[bytes] = ...,
        external_id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class CountryCreateResponse(_message.Message):
    __slots__ = ["response_standard", "country"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    COUNTRY_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    country: Country
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        country: _Optional[_Union[Country, _Mapping]] = ...,
    ) -> None: ...

class CountryReadRequest(_message.Message):
    __slots__ = ["group_by", "sort_by", "fields", "filter", "paginated", "id", "context"]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    SORT_BY_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    PAGINATED_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    group_by: _containers.RepeatedCompositeFieldContainer[_base_pb2.GroupBy]
    sort_by: _base_pb2.SortBy
    fields: _base_pb2.Fields
    filter: _base_pb2.Filter
    paginated: _base_pb2.Paginated
    id: str
    context: _base_pb2.Context
    def __init__(
        self,
        group_by: _Optional[_Iterable[_Union[_base_pb2.GroupBy, _Mapping]]] = ...,
        sort_by: _Optional[_Union[_base_pb2.SortBy, _Mapping]] = ...,
        fields: _Optional[_Union[_base_pb2.Fields, _Mapping]] = ...,
        filter: _Optional[_Union[_base_pb2.Filter, _Mapping]] = ...,
        paginated: _Optional[_Union[_base_pb2.Paginated, _Mapping]] = ...,
        id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class CountryReadResponse(_message.Message):
    __slots__ = ["response_standard", "meta_data", "countries"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    COUNTRIES_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    meta_data: _base_pb2.MetaData
    countries: _containers.RepeatedCompositeFieldContainer[Country]
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        meta_data: _Optional[_Union[_base_pb2.MetaData, _Mapping]] = ...,
        countries: _Optional[_Iterable[_Union[Country, _Mapping]]] = ...,
    ) -> None: ...

class CountryUpdateRequest(_message.Message):
    __slots__ = ["country", "context"]
    COUNTRY_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    country: Country
    context: _base_pb2.Context
    def __init__(
        self,
        country: _Optional[_Union[Country, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class CountryUpdateResponse(_message.Message):
    __slots__ = ["response_standard", "country"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    COUNTRY_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    country: Country
    def __init__(
        self,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
        country: _Optional[_Union[Country, _Mapping]] = ...,
    ) -> None: ...

class CountryDeleteRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: str
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[str] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class CountryDeleteResponse(_message.Message):
    __slots__ = ["response_standard"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    def __init__(self, response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...) -> None: ...



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.clients import type_document_pb2 as v1_dot_clients_dot_type__document__pb2


class TypeDocumentServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.TypeDocumentCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentCreate",
            request_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentCreateRequest.SerializeToString,
            response_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentCreateResponse.FromString,
        )
        self.TypeDocumentRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentRead",
            request_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentReadRequest.SerializeToString,
            response_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentReadResponse.FromString,
        )
        self.TypeDocumentUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentUpdate",
            request_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentUpdateResponse.FromString,
        )
        self.TypeDocumentDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentDelete",
            request_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentDeleteResponse.FromString,
        )


class TypeDocumentServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def TypeDocumentCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def TypeDocumentRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def TypeDocumentUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def TypeDocumentDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_TypeDocumentServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "TypeDocumentCreate": grpc.unary_unary_rpc_method_handler(
            servicer.TypeDocumentCreate,
            request_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentCreateRequest.FromString,
            response_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentCreateResponse.SerializeToString,
        ),
        "TypeDocumentRead": grpc.unary_unary_rpc_method_handler(
            servicer.TypeDocumentRead,
            request_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentReadRequest.FromString,
            response_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentReadResponse.SerializeToString,
        ),
        "TypeDocumentUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.TypeDocumentUpdate,
            request_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentUpdateRequest.FromString,
            response_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentUpdateResponse.SerializeToString,
        ),
        "TypeDocumentDelete": grpc.unary_unary_rpc_method_handler(
            servicer.TypeDocumentDelete,
            request_deserializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentDeleteRequest.FromString,
            response_serializer=v1_dot_clients_dot_type__document__pb2.TypeDocumentDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.clients.type_document.TypeDocumentService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class TypeDocumentService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def TypeDocumentCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentCreate",
            v1_dot_clients_dot_type__document__pb2.TypeDocumentCreateRequest.SerializeToString,
            v1_dot_clients_dot_type__document__pb2.TypeDocumentCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def TypeDocumentRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentRead",
            v1_dot_clients_dot_type__document__pb2.TypeDocumentReadRequest.SerializeToString,
            v1_dot_clients_dot_type__document__pb2.TypeDocumentReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def TypeDocumentUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentUpdate",
            v1_dot_clients_dot_type__document__pb2.TypeDocumentUpdateRequest.SerializeToString,
            v1_dot_clients_dot_type__document__pb2.TypeDocumentUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def TypeDocumentDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.clients.type_document.TypeDocumentService/TypeDocumentDelete",
            v1_dot_clients_dot_type__document__pb2.TypeDocumentDeleteRequest.SerializeToString,
            v1_dot_clients_dot_type__document__pb2.TypeDocumentDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



import ast
import operator

from omni.pro.protos.common import base_pb2
from sqlalchemy import and_, asc, create_engine, desc, not_, or_
from sqlalchemy.orm import aliased, scoped_session, sessionmaker
from sqlalchemy.sql import cast, operators
from sqlalchemy.sql.sqltypes import Enum, String


class SessionManager:
    def __init__(self, base_url):
        self.base_url = base_url
        self.engine = create_engine(self.base_url)
        self.session_factory = sessionmaker(bind=self.engine)
        self.Session = scoped_session(self.session_factory)

    def __enter__(self):
        # Esto dará una sesión específica para el hilo/contexto actual
        return self.Session()

    def __exit__(self, exc_type, exc_val, exc_tb):
        # Cuando el contexto se cierra, la sesión se cierra y elimina.
        if exc_tb is None:
            self.Session.commit()
        else:
            self.Session.rollback()
        self.Session.remove()


class PostgresDatabaseManager(SessionManager):
    """
    Handles database operations specific to Postgres using SQLAlchemy.
    Maneja operaciones de base de datos específicas para Postgres usando SQLAlchemy.

    Args:
    name (str): Name of the database.
                Nombre de la base de datos.
    host (str): Host of the database.
                Host de la base de datos.
    port (str): Port of the database.
                Puerto de la base de datos.
    user (str): User to connect to the database.
                Usuario para conectarse a la base de datos.
    password (str): Password to connect to the database.
                    Contraseña para conectarse a la base de datos.
    """

    DEFAULT_PAGE_SIZE = 10

    def __init__(self, name: str, host: str, port: str, user: str, password: str):
        """
        Initializes the PostgresDatabaseManager with the given database details.
        Inicializa el PostgresDatabaseManager con los detalles de base de datos proporcionados.

        Args:
        name (str): Name of the database.
                    Nombre de la base de datos.
        host (str): Host of the database.
                    Host de la base de datos.
        port (str): Port of the database.
                    Puerto de la base de datos.
        user (str): User to connect to the database.
                    Usuario para conectarse a la base de datos.
        password (str): Password to connect to the database.
                        Contraseña para conectarse a la base de datos.
        """
        self.name = name
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.base_url = f"postgresql://{user}:{password}@{host}:{port}/{name}"
        super().__init__(self.base_url)

    def get_db_connection(self):
        """
        Retrieves a connection to the Postgres database.
        Recupera una conexión a la base de datos Postgres.

        Returns:
        (obj): Database connection object.
               Objeto de conexión a la base de datos.
        """
        return self.engine.connect()

    def create_new_record(self, model, session, **kwargs):
        """
        Creates a new record in the database using the provided model and session.
        Crea un nuevo registro en la base de datos usando el modelo y la sesión proporcionados.

        Args:
        model (Base): The SQLAlchemy model where the record will be created.
                      El modelo SQLAlchemy donde se creará el registro.
        session (Session): An instance of the database session, likely from SQLAlchemy.
                           Una instancia de la sesión de base de datos, probablemente de SQLAlchemy.
        **kwargs: Attributes and values that will be used to initialize the new record of the model.
                  Atributos y valores que se utilizarán para inicializar el nuevo registro del modelo.

        Returns:
        (obj): The new model record after being added to the session.
               El nuevo registro del modelo después de ser añadido a la sesión.
        """
        record = model(**kwargs)
        record.create(session)
        return record

    def retrieve_record(self, model, session, filters: dict):
        """
        Retrieves a single database record based on provided filters.
        Recupera un único registro de la base de datos basado en los filtros proporcionados.

        Args:
        model (Base): The SQLAlchemy model to query.
                      El modelo SQLAlchemy a consultar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        filters (dict): A dictionary of attributes to filter the records by.
                        Un diccionario de atributos para filtrar los registros.

        Returns:
        (obj): The first record that matches the filters, or None if not found.
               El primer registro que coincida con los filtros, o None si no se encuentra.
        """
        return session.query(model).filter_by(**filters).first()

    def retrieve_records(self, model, session, filters: dict):
        """
        Retrieves database records based on provided filters.

        Args:
        model (Base): The SQLAlchemy model to query.
        session (Session): An instance of the database session.
        filters (dict): A dictionary of attributes to filter the records by.

        Returns:
        (objs): List of records that match with filters, or empty list if not found.
        """
        return session.query(model).filter_by(**filters).all()

    def retrieve_record_by_id(self, model, session, id: int):
        """
        Retrieves a single database record by its ID.
        Recupera un único registro de la base de datos por su ID.

        Args:
        model (Base): The SQLAlchemy model to query.
                      El modelo SQLAlchemy a consultar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        id (int): The ID of the record to retrieve.
                  El ID del registro a recuperar.

        Returns:
        (obj): The record with the given ID, or None if not found.
               El registro con el ID dado, o None si no se encuentra.
        """
        return session.query(model).get(id)

    def list_records(
        self,
        model,
        session,
        id: int,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
        paginated: base_pb2.Paginated,
    ):
        """
        Lists database records based on provided parameters.
        Lista registros de la base de datos basados en los parámetros proporcionados.

        Args:
        model (Base): The SQLAlchemy model to query.
                      El modelo SQLAlchemy a consultar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        id (int): The ID of a specific record to retrieve.
                  El ID de un registro específico para recuperar.
        fields (base_pb2.Fields): Fields to be returned in the response.
                                  Campos a ser devueltos en la respuesta.
        filter (base_pb2.Filter): Conditions to filter the list of records.
                                  Condiciones para filtrar la lista de registros.
        group_by (base_pb2.GroupBy): Conditions to group the list of records.
                                     Condiciones para agrupar la lista de registros.
        sort_by (base_pb2.SortBy): Conditions to sort the list of records.
                                   Condiciones para ordenar la lista de registros.
        paginated (base_pb2.Paginated): Conditions for pagination of the records.
                                        Condiciones para la paginación de los registros.

        Returns:
        (list): A list of records based on the provided parameters.
                Una lista de registros basados en los parámetros proporcionados.
        """
        # records = QueryBuilder.build_filter(model, session, id, fields, filter, group_by, sort_by, paginated)

        # Uso de la clase
        query = session.query(model)

        if filter.ListFields():
            # Uso de la clase
            expression = ast.literal_eval(filter.filter)  # Tu expresión en notación polaca inversa
            # converter = PolishNotationToSQLAlchemy(model, expression)
            # filter_condition, aliases = converter.convert()

            # Aplicar el filtro a la consulta
            # for alias in aliases.values():
            #     query = query.join(alias)

            query = self.parse_expression(expression, query, model)

        if id:
            query = query.filter(model.id == id)

        if fields.ListFields():
            query = query.with_entities(*[getattr(model, f) for f in fields.name_field])

        if sort_by.ListFields():
            order_by_fields = self.build_sort_by(model, sort_by)
            query = query.order_by(*order_by_fields)

        total = query.count()

        page = paginated.offset or 1
        limit = paginated.limit or self.DEFAULT_PAGE_SIZE
        offset = (page - 1) * limit
        query = query.offset(offset).limit(limit)

        return query.all(), total

    def build_sort_by(self, model, sort_by: base_pb2.SortBy):
        field = getattr(model, sort_by.name_field)
        if sort_by.type == sort_by.DESC:
            return [desc(field)]
        return [asc(field)]

    def update_record(self, model, session, model_id, update_dict):
        """
        Update a database record of the given model with the specified changes.
        Actualiza un registro de base de datos del modelo dado con los cambios especificados.

        Args:
        model (Base): The SQLAlchemy model to update.
                      El modelo SQLAlchemy a actualizar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        model_id: The ID of the record to update.
                  El ID del registro a actualizar.
        update_dict (dict): A dictionary of attributes and their new values.
                            Un diccionario de atributos y sus nuevos valores.

        Returns:
        (obj): The updated record, or None if not found.
               El registro actualizado, o None si no se encuentra.
        """
        record = session.query(model).get(model_id)
        if not record:
            return None

        for key, value in update_dict.items():
            setattr(record, key, value)

        record.update(session)
        return record

    def delete_record_by_id(self, model, session, model_id):
        """
        Delete a database record of the given model by its ID.
        Elimina un registro de base de datos del modelo dado por su ID.

        Args:
        model (Base): The SQLAlchemy model to delete from.
                      El modelo SQLAlchemy del que eliminar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        model_id: The ID of the record to delete.
                  El ID del registro a eliminar.

        Returns:
        (bool): True if the record was deleted, False otherwise.
                Verdadero si el registro fue eliminado, Falso en caso contrario.
        """
        record = session.query(model).filter_by(id=model_id).first()
        if record:
            success = record.delete(session)
            if success:
                return True
        return False

    def batch_upsert(self, model, session, data: list):
        """
        Batch upserts a list of records into the database.
        Actualiza por lotes una lista de registros en la base de datos.

        Args:
        model (Base): The SQLAlchemy model to upsert into.
                      El modelo SQLAlchemy para actualizar.
        session (Session): An instance of the database session.
                           Una instancia de la sesión de base de datos.
        data (list): A list of dictionaries containing the records to upsert.
                     Una lista de diccionarios que contienen los registros para actualizar.

        Returns:
        (list): A list of records that were upserted.
                Una lista de registros que fueron actualizados.
        """
        upsert_list = []
        for registro in data["models"]:
            registro = registro | {"tenant": data["context"]["tenant"], "updated_by": data["context"]["user"]}
            obj = session.query(model).filter_by(external_id=registro["external_id"]).first()
            if obj:
                for key, value in registro.items():
                    setattr(obj, key, value)
            else:
                obj = model(**registro)

            upsert_list.append(obj)

        session.bulk_save_objects(upsert_list, update_changed_only=True)
        session.commit()
        batch_upsert_process = set(
            session.query(model).filter(model.external_id.in_([result.external_id for result in upsert_list])).all()
        )

        session.close()

        return list(batch_upsert_process - set(upsert_list))

    def get_sqlalchemy_operator(self, op):
        """
        Devuelve el operador de SQLAlchemy correspondiente al operador en string.

        :param op: El operador en formato de string.
        :return: Operador de SQLAlchemy.
        """
        operator_mapping = {
            "=": operators.eq,
            "!=": operators.ne,
            "<": operators.lt,
            "<=": operators.le,
            ">": operators.gt,
            ">=": operators.ge,
            "like": operators.like_op,
            "ilike": operators.ilike_op,
            "in": operators.in_op,
            "not in": operators.notin_op,
            # Agrega más operadores según sea necesario
        }
        return operator_mapping.get(op)

    def resolve_field_and_joins(self, base_model, field_path):
        """
        Resuelve el campo y construye los joins necesarios.

        :param base_model: El modelo base de SQLAlchemy.
        :param field_path: El camino al campo, que puede incluir '__' para relaciones.
        :return: El atributo del modelo SQLAlchemy correspondiente y una lista de joins.
        """
        fields = field_path.split("__")
        model = base_model
        joins = []
        for field in fields[:-1]:
            relationship_property = getattr(model, field)
            model = relationship_property.property.mapper.class_
            joins.append(relationship_property)
        return getattr(model, fields[-1]), joins

    def parse_expression(self, expression, query, base_model):
        stack = []
        joins = []
        for item in reversed(expression):
            if isinstance(item, tuple):
                field_path, op, value = item
                field, join_path = self.resolve_field_and_joins(base_model, field_path)

                joins.extend(join_path)

                operator_func = self.get_sqlalchemy_operator(op)

                if not operator_func:
                    raise ValueError(f"Operador desconocido: {op}")

                # Casting a texto si es necesario
                if isinstance(field.type, Enum) and op in ["like", "ilike"]:
                    field = cast(field, String)

                clause = operator_func(field, value)
                stack.append(clause)
            elif item in ["and", "or"]:
                right = stack.pop()
                left = stack.pop()
                if item == "and":
                    stack.append(and_(left, right))
                else:
                    stack.append(or_(left, right))

        if len(stack) != 1:
            raise ValueError("Expresión inválida")

        for join in joins:
            query = query.join(join)
        return query.filter(stack[0])


class PolishNotationToSQLAlchemy:
    def __init__(self, model, expression):
        self.model = model
        self.expression = expression
        self.aliases = {}

    def is_logical_operator(self, token):
        return token in ["and", "or", "not"]

    def is_comparison_operator(self, token):
        return token in ["=", "!=", "like", "!like", "in", "nin", ">", "<", ">=", "<="]

    def is_tuple(self, token):
        return isinstance(token, tuple) and len(token) == 3

    def get_field(self, model, field_path):
        fields = field_path.split("__")
        for field in fields[:-1]:  # process relationships
            relationship = getattr(model, field, None)
            if relationship is None:
                raise AttributeError(f"No such relationship {field} on {model}")
            if relationship in self.aliases:
                model = self.aliases[relationship]
            else:
                alias = aliased(relationship.property.mapper.class_)
                self.aliases[relationship] = alias
                model = alias
        return getattr(model, fields[-1], None), model

    def create_filter(self, model, field_path, operator, value):
        field, model = self.get_field(model, field_path)
        if operator == "=":
            return field == value
        elif operator == "!=":
            return field != value
        elif operator == "like":
            return field.like(value)
        elif operator == "!like":
            return not_(field.like(value))
        elif operator == "in":
            return field.in_(value)
        elif operator == "nin":
            return not_(field.in_(value))
        elif operator == ">":
            return field > value
        elif operator == "<":
            return field < value
        elif operator == ">=":
            return field >= value
        elif operator == "<=":
            return field <= value
        else:
            raise ValueError(f"Unknown operator: {operator}")

    def convert(self):
        operand_stack = []

        for token in reversed(self.expression):
            if self.is_logical_operator(token):
                operands = [operand_stack.pop() for _ in range(2)]
                if token == "and":
                    operand_stack.append(and_(*operands))
                elif token == "or":
                    operand_stack.append(or_(*operands))
                elif token == "not":
                    operand_stack.append(not_(operands[0]))
            elif self.is_tuple(token):
                field_path, operator, value = token
                operand_stack.append(self.create_filter(self.model, field_path, operator, value))
            else:
                raise ValueError(f"Unexpected token: {token}")

        if len(operand_stack) != 1:
            raise ValueError("The expression does not resolve to a single operand.")

        return operand_stack[0], self.aliases


class QueryBuilder:
    filter_ops = {
        "=": operator.eq,
        "<": operator.lt,
        "<=": operator.le,
        ">": operator.gt,
        ">=": operator.ge,
        "!=": operator.ne,
        "in": lambda x, y: x.in_(y),
        "and": operator.and_,
        "or": operator.or_,
        "not": lambda x: ~(x),
    }
    DEFAULT_PAGE_SIZE = 10

    @classmethod
    def pre_to_in(cls, filters: base_pb2.Filter) -> list:
        str_filter = filters.filter.replace("true", "True").replace("false", "False")
        expression = ast.literal_eval(str_filter)
        stack = []
        result = []
        for item in expression:
            if isinstance(item, tuple):
                if stack:
                    result.append(item)
                    result.append(stack.pop())
                else:
                    result.append(item)
            else:
                stack.append(item)
        return result

    @classmethod
    def build_filter(
        cls,
        model,
        session,
        id: int,
        fields: base_pb2.Fields,
        filter: base_pb2.Filter,
        group_by: base_pb2.GroupBy,
        sort_by: base_pb2.SortBy,
        paginated: base_pb2.Paginated,
    ):
        query = session.query(model)

        if id:
            query = query.filter(model.id == id)

        if fields.ListFields():
            query = query.with_entities(*[getattr(model, f) for f in fields.name_field])

        if filter.ListFields():
            filter_custom = cls.pre_to_in(filter)
            query_filter = cls.build_query(model, filter_custom)
            query = query.filter(query_filter)

        if group_by:
            group_by_fields = cls.build_group_by(model, group_by)
            query = query.group_by(*group_by_fields)

        if sort_by.ListFields():
            order_by_fields = cls.build_sort_by(model, sort_by)
            query = query.order_by(*order_by_fields)

        """
            Retrieves a paginated list from a given query.

            Args:
            
            query (obj): The query object that needs pagination.
            paginated (obj): Contains pagination parameters such as offset and limit.

            Returns:
            
            tuple: A tuple containing the paginated list and the total count.
            
        """

        total = query.count()

        if paginated.ListFields():
            page = paginated.offset or 1
            limit = paginated.limit or cls.DEFAULT_PAGE_SIZE
            offset = (page - 1) * limit
            query = query.offset(offset).limit(limit)

        return query.all(), total

    @classmethod
    def build_query(cls, model, filters: list):
        query = None

        filter_operator = None
        for item in filters:
            if isinstance(item, str):
                filter_operator = item.lower()
                if filter_operator not in cls.filter_ops:
                    raise ValueError(f"Invalid filter operator: {filter_operator}")
            elif isinstance(item, tuple) and len(item) == 3:
                field, op, value = item
                if "." in field:
                    related, field = field.split(".")
                    related_model = getattr(model, related).property.argument
                    related_field = getattr(related_model, field)
                    related_query = cls.filter_ops[op](related_field, value)
                    query = cls.filter_ops[op](query, related_query) if query else related_query
                else:
                    field = getattr(model, field)
                    field_query = cls.filter_ops[op](field, value)
                    query = cls.filter_ops[filter_operator](query, field_query) if filter_operator else field_query
            else:
                raise ValueError(f"Invalid filter item: {item}")

        return query

    @classmethod
    def build_sort_by(cls, model, sort_by: base_pb2.SortBy):
        field = getattr(model, sort_by.name_field)
        if sort_by.type == sort_by.DESC:
            return [desc(field)]
        return [asc(field)]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/redistribution_order.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n#v1/rules/redistribution_order.proto\x12.pro.omni.oms.api.v1.rules.redistribution_order\x1a\x11\x63ommon/base.proto\x1a\x1cgoogle/protobuf/struct.proto"\x96\x01\n\x13RedistributionOrder\x12\n\n\x02id\x18\x01 \x01(\t\x12\x32\n\x11\x64istribution_data\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x12?\n\x0cobject_audit\x18\x03 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"j\n RedistributionOrderCreateRequest\x12\x0e\n\x06\x61\x63tion\x18\x01 \x01(\x08\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe2\x01\n!RedistributionOrderCreateResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x61\n\x14redistribution_order\x18\x02 \x01(\x0b\x32\x43.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrder\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xfc\x02\n\x1eRedistributionOrderReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x8b\x02\n\x1fRedistributionOrderReadResponse\x12\x62\n\x15redistribution_orders\x18\x01 \x03(\x0b\x32\x43.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrder\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x03 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData"f\n RedistributionOrderDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x7f\n!RedistributionOrderDeleteResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xdf\x04\n\x1aRedistributionOrderService\x12\xc0\x01\n\x19RedistributionOrderAction\x12P.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrderCreateRequest\x1aQ.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrderCreateResponse\x12\xba\x01\n\x17RedistributionOrderRead\x12N.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrderReadRequest\x1aO.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrderReadResponse\x12\xc0\x01\n\x19RedistributionOrderDelete\x12P.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrderDeleteRequest\x1aQ.pro.omni.oms.api.v1.rules.redistribution_order.RedistributionOrderDeleteResponseb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.redistribution_order_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_REDISTRIBUTIONORDER"]._serialized_start = 137
    _globals["_REDISTRIBUTIONORDER"]._serialized_end = 287
    _globals["_REDISTRIBUTIONORDERCREATEREQUEST"]._serialized_start = 289
    _globals["_REDISTRIBUTIONORDERCREATEREQUEST"]._serialized_end = 395
    _globals["_REDISTRIBUTIONORDERCREATERESPONSE"]._serialized_start = 398
    _globals["_REDISTRIBUTIONORDERCREATERESPONSE"]._serialized_end = 624
    _globals["_REDISTRIBUTIONORDERREADREQUEST"]._serialized_start = 627
    _globals["_REDISTRIBUTIONORDERREADREQUEST"]._serialized_end = 1007
    _globals["_REDISTRIBUTIONORDERREADRESPONSE"]._serialized_start = 1010
    _globals["_REDISTRIBUTIONORDERREADRESPONSE"]._serialized_end = 1277
    _globals["_REDISTRIBUTIONORDERDELETEREQUEST"]._serialized_start = 1279
    _globals["_REDISTRIBUTIONORDERDELETEREQUEST"]._serialized_end = 1381
    _globals["_REDISTRIBUTIONORDERDELETERESPONSE"]._serialized_start = 1383
    _globals["_REDISTRIBUTIONORDERDELETERESPONSE"]._serialized_end = 1510
    _globals["_REDISTRIBUTIONORDERSERVICE"]._serialized_start = 1513
    _globals["_REDISTRIBUTIONORDERSERVICE"]._serialized_end = 2120
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/appointment.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2
from omni.pro.protos.v1.rules import delivery_method_pb2 as v1_dot_rules_dot_delivery__method__pb2
from omni.pro.protos.v1.rules import warehouse_pb2 as v1_dot_rules_dot_warehouse__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1av1/rules/appointment.proto\x12%pro.omni.oms.api.v1.rules.appointment\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1ev1/rules/delivery_method.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x18v1/rules/warehouse.proto"\x88\x01\n\x10\x41ppointmentLines\x12\x12\n\nhour_start\x18\x01 \x01(\t\x12\x10\n\x08hour_end\x18\x02 \x01(\t\x12\x15\n\rorder_numbers\x18\x03 \x01(\x05\x12\x1d\n\x15\x61ppointment_available\x18\x04 \x01(\x05\x12\x0c\n\x04\x63ids\x18\x05 \x03(\t\x12\n\n\x02id\x18\x06 \x01(\t"\xff\x02\n\x0b\x41ppointment\x12\n\n\x02id\x18\x01 \x01(\t\x12\x41\n\twarehouse\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.v1.rules.warehouse.Warehouse\x12I\n\x06method\x18\x03 \x01(\x0b\x32\x39.pro.omni.oms.api.v1.rules.delivery_method.DeliveryMethod\x12\x0c\n\x04\x64\x61te\x18\x04 \x01(\t\x12\x46\n\x05lines\x18\x05 \x03(\x0b\x32\x37.pro.omni.oms.api.v1.rules.appointment.AppointmentLines\x12*\n\x06\x61\x63tive\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x07 \x01(\t\x12?\n\x0cobject_audit\x18\x08 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xe6\x01\n\x18\x41ppointmentCreateRequest\x12\x14\n\x0cwarehouse_id\x18\x01 \x01(\x05\x12\x11\n\tmethod_id\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61te\x18\x03 \x01(\t\x12\x46\n\x05lines\x18\x04 \x03(\x0b\x32\x37.pro.omni.oms.api.v1.rules.appointment.AppointmentLines\x12\x13\n\x0b\x65xternal_id\x18\x05 \x01(\t\x12\x36\n\x07\x63ontext\x18\x06 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xaf\x01\n\x19\x41ppointmentCreateResponse\x12G\n\x0b\x61ppointment\x18\x01 \x01(\x0b\x32\x32.pro.omni.oms.api.v1.rules.appointment.Appointment\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xf4\x02\n\x16\x41ppointmentReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe9\x01\n\x17\x41ppointmentReadResponse\x12H\n\x0c\x61ppointments\x18\x01 \x03(\x0b\x32\x32.pro.omni.oms.api.v1.rules.appointment.Appointment\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x9b\x01\n\x18\x41ppointmentUpdateRequest\x12G\n\x0b\x61ppointment\x18\x01 \x01(\x0b\x32\x32.pro.omni.oms.api.v1.rules.appointment.Appointment\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xaf\x01\n\x19\x41ppointmentUpdateResponse\x12G\n\x0b\x61ppointment\x18\x01 \x01(\x0b\x32\x32.pro.omni.oms.api.v1.rules.appointment.Appointment\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"^\n\x18\x41ppointmentDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"f\n\x19\x41ppointmentDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"`\n\x18SearchAppointmentRequest\x12\x0c\n\x04\x63ids\x18\x01 \x03(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x8d\x01\n\x19SearchAppointmentResponse\x12%\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"m\n\x19\x43onfirmAppointmentRequest\x12\x0b\n\x03\x61id\x18\x01 \x01(\t\x12\x0b\n\x03\x63id\x18\x02 \x01(\t\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x89\x01\n\x1a\x43onfirmAppointmentResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xb3\x07\n\x12\x41ppointmentService\x12\x98\x01\n\x11\x41ppointmentCreate\x12?.pro.omni.oms.api.v1.rules.appointment.AppointmentCreateRequest\x1a@.pro.omni.oms.api.v1.rules.appointment.AppointmentCreateResponse"\x00\x12\x92\x01\n\x0f\x41ppointmentRead\x12=.pro.omni.oms.api.v1.rules.appointment.AppointmentReadRequest\x1a>.pro.omni.oms.api.v1.rules.appointment.AppointmentReadResponse"\x00\x12\x98\x01\n\x11\x41ppointmentUpdate\x12?.pro.omni.oms.api.v1.rules.appointment.AppointmentUpdateRequest\x1a@.pro.omni.oms.api.v1.rules.appointment.AppointmentUpdateResponse"\x00\x12\x98\x01\n\x11\x41ppointmentDelete\x12?.pro.omni.oms.api.v1.rules.appointment.AppointmentDeleteRequest\x1a@.pro.omni.oms.api.v1.rules.appointment.AppointmentDeleteResponse"\x00\x12\x98\x01\n\x11SearchAppointment\x12?.pro.omni.oms.api.v1.rules.appointment.SearchAppointmentRequest\x1a@.pro.omni.oms.api.v1.rules.appointment.SearchAppointmentResponse"\x00\x12\x9b\x01\n\x12\x43onfirmAppointment\x12@.pro.omni.oms.api.v1.rules.appointment.ConfirmAppointmentRequest\x1a\x41.pro.omni.oms.api.v1.rules.appointment.ConfirmAppointmentResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.appointment_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _globals["_APPOINTMENTLINES"]._serialized_start = 209
    _globals["_APPOINTMENTLINES"]._serialized_end = 345
    _globals["_APPOINTMENT"]._serialized_start = 348
    _globals["_APPOINTMENT"]._serialized_end = 731
    _globals["_APPOINTMENTCREATEREQUEST"]._serialized_start = 734
    _globals["_APPOINTMENTCREATEREQUEST"]._serialized_end = 964
    _globals["_APPOINTMENTCREATERESPONSE"]._serialized_start = 967
    _globals["_APPOINTMENTCREATERESPONSE"]._serialized_end = 1142
    _globals["_APPOINTMENTREADREQUEST"]._serialized_start = 1145
    _globals["_APPOINTMENTREADREQUEST"]._serialized_end = 1517
    _globals["_APPOINTMENTREADRESPONSE"]._serialized_start = 1520
    _globals["_APPOINTMENTREADRESPONSE"]._serialized_end = 1753
    _globals["_APPOINTMENTUPDATEREQUEST"]._serialized_start = 1756
    _globals["_APPOINTMENTUPDATEREQUEST"]._serialized_end = 1911
    _globals["_APPOINTMENTUPDATERESPONSE"]._serialized_start = 1914
    _globals["_APPOINTMENTUPDATERESPONSE"]._serialized_end = 2089
    _globals["_APPOINTMENTDELETEREQUEST"]._serialized_start = 2091
    _globals["_APPOINTMENTDELETEREQUEST"]._serialized_end = 2185
    _globals["_APPOINTMENTDELETERESPONSE"]._serialized_start = 2187
    _globals["_APPOINTMENTDELETERESPONSE"]._serialized_end = 2289
    _globals["_SEARCHAPPOINTMENTREQUEST"]._serialized_start = 2291
    _globals["_SEARCHAPPOINTMENTREQUEST"]._serialized_end = 2387
    _globals["_SEARCHAPPOINTMENTRESPONSE"]._serialized_start = 2390
    _globals["_SEARCHAPPOINTMENTRESPONSE"]._serialized_end = 2531
    _globals["_CONFIRMAPPOINTMENTREQUEST"]._serialized_start = 2533
    _globals["_CONFIRMAPPOINTMENTREQUEST"]._serialized_end = 2642
    _globals["_CONFIRMAPPOINTMENTRESPONSE"]._serialized_start = 2645
    _globals["_CONFIRMAPPOINTMENTRESPONSE"]._serialized_end = 2782
    _globals["_APPOINTMENTSERVICE"]._serialized_start = 2785
    _globals["_APPOINTMENTSERVICE"]._serialized_end = 3732
# @@protoc_insertion_point(module_scope)



from botocore.config import Config
from omni.pro.aws.client import AWSClient


class AWSS3Client(AWSClient):
    def __init__(
        self,
        bucket_name: str,
        region_name: str,
        aws_access_key_id: str,
        aws_secret_access_key: str,
        allowed_files: list,
        **kwargs,
    ) -> None:
        """
        Initializes a client for interacting with Amazon S3.

        :param bucket_name: str
        The name of the S3 bucket the client will access.

        :param region_name: str
        The region where the S3 bucket is hosted.

        :param aws_access_key_id: str
        The AWS access key ID.

        :param aws_secret_access_key: str
        The AWS secret access key.

        Additional kwargs are passed to the base class AWSClient constructor, allowing further configuration.
        """
        kwargs["config"] = Config(
            region_name=region_name, signature_version="v4", retries={"max_attempts": 10, "mode": "standard"}
        )
        self.bucket_name = bucket_name
        self.allowed_files = allowed_files
        super().__init__("s3", region_name, aws_access_key_id, aws_secret_access_key, **kwargs)

    def download_file(self, object_name: str, file_path: str):
        """
        Downloads a file from an S3 bucket.

        :param object_name: str
        The name of the object in S3 to be downloaded.

        :param file_path: str
        The path of the local file where the downloaded object will be saved.

        :return: None
        """
        result = self.client.download_file(self.bucket_name, object_name, file_path)
        return result

    def upload_file(self, object_name: str, file_path: str):
        """
        Uploads a file to an S3 bucket.

        :param file_path: str
        The path of the local file to be uploaded.

        :param object_name: str
        The name of the object in S3 to be uploaded.

        :return: None
        """
        self.client.upload_file(file_path, self.bucket_name, object_name)
        return object_name

    def generate_presigned_post(self, object_name: str):
        """
        Generate presigned post to upload file to an S3 bucket.

        :param object_name: str
        The name of the object in S3 to be uploaded.

        :return: presigned url
        """
        return self.client.generate_presigned_post(self.bucket_name, object_name, ExpiresIn=3600)

    def generate_presigned_url(self, object_name: str):
        """
        Generate presigned url to download file from an S3 bucket.

        :param object_name: str
        The name of the object in S3 to be uploaded.

        :return: presigned url
        """
        return self.client.generate_presigned_url("get_object", Params={"Bucket": self.bucket_name, "Key": object_name})
